##############################################################################
# base module with assorted ctypes utilities
# 
# Part of the python bindings for teem, by Carlos Scheidegger
# teem is developed by Gordon Kindlmann, and available at
# http://teem.sourceforge.net

from ctypes import *
from ctypes.util import find_library

teem_path = find_library('teem')
if teem_path is None:
    raise Exception("Could not locate teem shared library. Make sure you have built teem with shared library support.")
libteem = cdll.LoadLibrary(teem_path)
libc = cdll.LoadLibrary(find_library('c'))

c_intp = POINTER(c_int)
c_voidpp = POINTER(c_voidp)
c_uintp = POINTER(c_uint)
c_charpp = POINTER(c_char_p)
c_doublep = POINTER(c_double)
c_floatp = POINTER(c_float)
c_size_tp = POINTER(c_size_t)

class FILE(Structure):
    pass

FILE_p = POINTER(FILE)

##############################################################################
# assorted ctype hackery

def from_file(f):
    return cast(pythonapi.PyFile_AsFile(py_object(f)), FILE_p)

ctypes_base_class = type(c_int(0)).__mro__[-2]
# this is just a feeble attempt at robustness
assert ctypes_base_class.__name__ == '_CData'

##############################################################################

def ctypes_new_fun(library, funName, retType, params):
    f = getattr(library, funName)
    f.argtypes = params
    f.restype = retType
    return f

def ctypes_new_fun_retnewstr(library, funName, params):
    f = getattr(library, funName)
    f.argtypes = params
    f.restype = c_void_p
    def _callable(*args):
        r = f(*args)
        s = string_at(r)
        libc.free(r)
        return s
    return _callable
    
def ctype_simple_ro_property(vname):
    def _reader(self):
        return getattr(self._ctype.contents, vname)
    return property(_reader)

def ctype_simple_rw_property(vname):
    def _reader(self):
        return getattr(self._ctype.contents, vname)
    def _writer(self, value):
        setattr(self._ctype.contents, vname, value)
    return property(_reader, _writer)

