#!/usr/bin/env python

# estimate the histogram using kernel density estimation by sampling a
# bunch of random points in the volume

import sys
import teem.gage as gage
import teem.nrrd as nrrd
import numpy
import time
import scipy
import scipy.stats
from pylab import *

def usage():
    print "%s nrrd_in num_samples" % sys.argv[0]
    sys.exit(1)

def sample_volume(nin, sample_count, mn_range, mx_range):
    context = gage.GageContext()
    gage.check_volume(gage.Kind.Scalar, nin)
    pvl = gage.GagePerVolume(context, nin, gage.Kind.Scalar)
    context.attach(pvl)
    context.set_kernel(gage.Kernel.K00, nrrd.Kernel.BCCubic, [1.0, 0.0, 0.5])
    context.set_kernel(gage.Kernel.K11, nrrd.Kernel.BCCubicD, [1.0, 0.0, 0.5])
    context.query_item_on(pvl, gage.ScalarMeasurement.Value)
    context.update()
    p = context.answer_pointer(pvl, gage.ScalarMeasurement.Value)
    result = numpy.zeros(sample_count)
    sx = nin.axis[0].size
    sy = nin.axis[1].size
    sz = nin.axis[2].size
    rpa = numpy.random.random((3, sample_count)) # give it some slack
    rpa[0] *= sx
    rpa[1] *= sy
    rpa[2] *= sz
    rpa[0] -= 0.5
    rpa[1] -= 0.5
    rpa[2] -= 0.5
    random_points = nrrd.Nrrd.from_array(rpa.transpose())
    n = context.probe_many(random_points)[0][gage.ScalarMeasurement.Value]
    result = numpy.array(n).reshape((sample_count,))
    return result[(result >= mn_range) & (result <= mx_range)]

def iqr(lst):
    s = sorted(lst)
    return s[-len(s) / 4] - s[len(s)/4]

def optimal_bin_width(lst):
    return 2 * iqr(lst) * len(lst) ** (-0.33333)

if __name__ == '__main__':
    try:
        nin = nrrd.Nrrd.load(sys.argv[1])
        sample_count = int(sys.argv[2])
    except:
        usage()
    samples = sample_volume(nin, sample_count, 1, 255)
    bin_count = 255.0 / optimal_bin_width(samples)
    print optimal_bin_width(samples)
    print "foo"
    x = scipy.linspace(0, 255, 256)
    figure()
    hist(samples, bins=255/optimal_bin_width(samples), normed=True)
    show()
    
